GemList = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")
local L = AceLibrary("AceLocale-2.2"):new("GemList")
local T = AceLibrary("Gratuity-2.0");


local chatprefix2 = "<GemList>"
local LOCKED = 0
local vmajor, vminor = "GemList-0.4", "$Rev: 27265 $"

GemList.options = { 
    type="group",
    args = {
		inc = {
			type = "toggle",
			name = L["Show incoming requests"],
			desc = L["Show incoming requests"],
			get = function() return GemList.db.account.showincmessages end,
			set = function(v) GemList.db.account.showincmessages = v end,
    	},
		out = {
			type="toggle",
			name = L["Show your replies"],
			desc = L["Show your replies"],
			get = function() return GemList.db.account.showexmessages end,
			set = function(v) GemList.db.account.showexmessages = v end,
		},
		autoscan = {
			type = "toggle",
			name = L["Autoscan"],
			desc = L["Automatically scan your Jewelcrafting panel when it opens"],
			get = function() return GemList.db.account.autoscan end,
			set = "ToggleAutoScan",
		},
		desc = {
			type="toggle",
			name = L["Search description"],
			desc = L["Also search gem description for a match"],
			get = function() return GemList.db.account.searchdesc end,
			set = function(v) GemList.db.account.searchdesc = v end,
		},
		scan = {
			type="execute",
			name = L["Manual scan (open your Jewelcrafting panel first)"],
			desc = L["Manual scan (open your Jewelcrafting panel first)"],
			func = "Scan",
			order = 101,
		},
		tlock = {
			type="range",
			name = L["Lock time between 2 requests (antispam)"],
			desc = L["Lock time between 2 requests (antispam)"],
			get = function() return GemList.db.account.TIMELOCK end,
			set = function(v) GemList.db.account.TIMELOCK = v end,
			min = 1,
			max = 30,
			step = 1,
		},
		chatprefix = {
			type = "text",
			name = L["Chat prefix to trigger GemList"],
			desc = L["Chat prefix to trigger GemList"],
			get = "getChatPrefix",
			set = function(v) GemList.db.account.chatprefix = v end,
			usage = "<prefix>"
		},
		send = {
			type = "text",
			name = L["Send a GemList to a player"],
			desc = L["Send a GemList to a player"],
			get = false,
			set = "Send",
			usage = "<playername> \[query\]"
		},
		version = {
			type = "execute",
			name = "Version: ",
			desc = vmajor .. " " .. vminor,
			func = function() GemList:Print(vmajor .. " " .. vminor) end,
			order = 102,
		},
			
	}
}

function GemList:OnInitialize()
	self:Reset()
	self:RegisterDB("GemListDB")
	self:RegisterDefaults("account", {
		Gems = {},
		Gemsname = {},
		Gemsdesc = {},
		showincmessages = false,
		showexmessages = false,
		searchdesc = true,
		autoscan = true,
		TIMELOCK = 3,
		chatprefix = "!gem"
		}
	)  
	
	self:RegisterChatCommand({"/glist", "/gemlist"}, self.options)
end

function GemList:OnEnable()
	self:Hook("ChatFrame_MessageEventHandler", "WhisperHandler", true)
	self:RegisterEvent("CHAT_MSG_WHISPER")
	if self.db.account.autoscan then 
		self:RegisterEvent("TRADE_SKILL_SHOW") 
	end
end

function GemList:OnDisable()
	self:Reset()
	self:UnhookAll()
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:UnregisterEvent("TRADE_SKILL_SHOW")
end

function GemList:Send(arg1)

	local argum = self:GetArgs(arg1, " ")
	local sender = argum[1]
	
	table.remove(argum, 1)
	
	if table.getn(argum) == 0 then 
		self:printHelp(sender)			
		return 
	end
	
	self:Handle(argum, sender)
end


function GemList:Reset()
	LOCKED = 0
end

function GemList:WhisperHandler(event) 
	if arg1 and arg1 ~= "" then
		
		if (string.sub(arg1, 1, string.len(self:getChatPrefix())) == self:getChatPrefix() and 
		not self.db.account.showincmessages and 
		event == "CHAT_MSG_WHISPER") then
			return
		end
		
		if (string.sub(arg1, 1, string.len(chatprefix2)) == chatprefix2 and 
		not self.db.account.showexmessages and 
		event == "CHAT_MSG_WHISPER_INFORM") then
			return
		end
	end
	return self.hooks["ChatFrame_MessageEventHandler"](event)
end

function GemList:printHelp(sender)
	local numLines = tonumber(L["numHelpLines"])
	self:Respond(L["usage1"], sender)
	for i = 2, numLines do 
		self:Respond(self:getChatPrefix() .. L["usage"..i], sender)
	end
	
	self:Respond(L["examples1"], sender)
	for i = 2, numLines do 
		self:Respond(self:getChatPrefix() .. L["examples"..i], sender)
	end
	 
	if self.db.account.searchdesc then 
		self:Respond(L["Searching gem description is currently enabled."], sender)
	else 
		self:Respond(L["Searching gem description is currently disabled."], sender)
	end
end

function GemList:CHAT_MSG_WHISPER(...)

    local argum = self:GetArgs(select(1, ...), " ")
    local sender = select(2, ...);
    local tag = select(6, ...);

    if argum[1] == self:getChatPrefix() or strfind(argum[1], "!gem") then 
        if tag == L["AFK"] or tag == L["DND"] then 

            self:Respond(L["First clear your tag: "] .. tag, sender) 
            return 
        end
        
        table.remove(argum, 1)

        if table.getn(argum) == 0 then 
            self:printHelp(sender)            
            return 
        end
        
        self:Handle(argum, sender)
    end
end

function GemList:GetArgs(text, separator)
   
	-- capture a link
	local itemName = string.match(text, "\124h.(.+).\124h");
	text = string.gsub(text, "\124c.+\124r", "");
  
    -- copy arguments to table
    local args = {};
    local i = 0;
	
	for value in string.gmatch(text, "[^"..separator.."]+") do
        i = i + 1;
        args[i] = string.lower(value);
    end    

	if itemName then 
        args[i+1] = itemName;
    end

    return args
end

function GemList:Handle(argum, sender)
	if table.getn(self.db.account.Gems) == 0 then 
		self:Respond(L["Database is currently empty."], sender) 
		self:Print(sender..L[" wants to use GemList but the Jewelcrafting panel has not yet been opened."])
		return 
	end
	
	local searchresult={}
	
	for i in pairs(self.db.account.Gems) do
		local paf = 0
		local GemName = string.lower(self.db.account.GemsName[i])
		local GemDescription = string.lower(self.db.account.GemsDescription[i])
		local GemColor = string.lower(self.db.account.GemsColor[i])
		local GemReagent = string.lower(self.db.account.GemsReagent[i])
		
		for j in pairs(argum) do
			local arg = string.lower(argum[j]);
			
			if strfind(GemName, arg) then 
				paf = paf + 1 
			elseif self.db.account.searchdesc and strfind(GemDescription, arg) then 
				paf = paf + 1
			elseif strfind(GemColor, arg) then
				paf = paf + 1
			elseif strfind(GemReagent, arg) then
				paf = paf + 1
			end
		end
		 
		if paf == table.getn(argum) then 
			table.insert(searchresult, self.db.account.Gems[i]) 
		end	
	end
	
	if table.getn(searchresult) == 0 then 
		self:Respond(L["Nothing found."], sender)
	else 
		if LOCKED == 0 then
			LOCKED = 1
			
			for i in pairs(searchresult) do 
				self:Respond(searchresult[i], sender) 
			end 
			
			self:ScheduleEvent(function() LOCKED=0 return end, self.db.account.TIMELOCK)
		else 
			self:Respond(L["Wait ... ("]..self.db.account.TIMELOCK..L[" seconds lock (antispam))"], sender)
		end
	end
end
function GemList:Respond(message, sender)
	SendChatMessage(chatprefix2.." "..message, "WHISPER", nil, sender)
end

function GemList:TRADE_SKILL_SHOW()
	if self.db.account.autoscan and self:isJewelcraftingPanel() then 
		self:Scan()
	end
end

function GemList:isJewelcraftingPanel()
	if not TradeSkillFrameTitleText then 
		return false
	end
		
	local frameTitle = TradeSkillFrameTitleText:GetText();
	if not strfind(frameTitle, L["Jewelcrafting"]) then 
		return false
	end
	return true
end


function GemList:ToggleAutoScan()
	self.db.account.autoscan = not self.db.account.autoscan
	if self.db.account.autoscan then 
		self:RegisterEvent("TRADE_SKILL_SHOW")
	else
		self:UnregisterEvent("TRADE_SKILL_SHOW") 
	end
end

function GemList:GetFirstGemIndex()
-- finds the Gems category and expands it if needed
	for i = 1, GetNumTradeSkills() do
		local skillName, skillType, isExpanded;
		skillName, skillType, _, isExpanded = GetTradeSkillInfo(i);
		if skillType == "header" and skillName == L["Gems"] then
			if not isExpanded then 
				ExpandTradeSkillSubClass(i);
			end
			return i+1;
		end
	end
	return nil
end

function GemList:getChatPrefix()
	return string.lower(self.db.account.chatprefix)
end

function GemList:getReagents(id)
	local numReagents = GetTradeSkillNumReagents(id);
	local reagents = "";
	
	for i = 1, numReagents do
		reagents = reagents .. GetTradeSkillReagentInfo(id, i) .. ", ";
	end

	return reagents
end
	

function GemList:Scan()

	if not self:isJewelcraftingPanel() then 
		self:Print(L["Open your Jewelcrafting panel first"]) 
		return 
	end
	
	self.db.account.Gems = {}
	self.db.account.GemsName = {}
	self.db.account.GemsDescription = {}
	self.db.account.GemsColor = {}
	self.db.account.GemsReagent = {}
	
	local firstGem = self:GetFirstGemIndex();
	if not firstGem return end
	for i = firstGem, GetNumTradeSkills() do
		
		local skillName, skillType = GetTradeSkillInfo(i)
		if skillType ~= "header" and skillName then
			
			T:SetTradeSkillItem(i)
			local color = T:GetLine(T:NumLines())
			local description = T:GetLine(2)
			table.insert(self.db.account.Gems, GetTradeSkillItemLink(i))
			table.insert(self.db.account.GemsName, skillName)
			table.insert(self.db.account.GemsColor, color)
		 	table.insert(self.db.account.GemsDescription, description)
			table.insert(self.db.account.GemsReagent, self:getReagents(i))
		else
			return
		end
		
	end
end

function GemList:test()
--dont look at this 

	self.db.account.Gems = {}
	self.db.account.GemsName = {}
	self.db.account.GemsDescription = {}
	self.db.account.GemsColor = {}
	self.db.account.GemsReagent = {}
	
	
	for i = GetFirstTradeSkill(), 3 do
		
		local skillName, skillType = GetTradeSkillInfo(i)
		if skillType ~= "header" and skillName then
			
			T:SetTradeSkillItem(i)
			local color = T:GetLine(T:NumLines())
			local description = T:GetLine(2)
			table.insert(self.db.account.Gems, GetTradeSkillItemLink(i))
			table.insert(self.db.account.GemsName, skillName)
			table.insert(self.db.account.GemsColor, color)
		 	table.insert(self.db.account.GemsDescription, description)
			table.insert(self.db.account.GemsReagent, self:getReagents(i))
		else
			return
		end
		
	end

GemList:Print(GemList.db.account.Gems[1])
GemList:Print(GemList.db.account.GemsName[1])
GemList:Print(GemList.db.account.GemsColor[1])
GemList:Print(GemList.db.account.GemsReagent[1])
end
